<?php


class cpsmart_otp_ajax
{
    private $cp_otp_options;
    private $public = [
        'facebook_verify' => ['cp-fb-verify', 'cp-fb-verify'],
        'woocommerce_verify' => ['cpotp-woo', 'cpotp-woo'],
        'woocommerce_download_verify' => ['cpotp-woo-download', 'cpotp-woo-download'],
        'cf7_verify' => ['cpotp-cf7', 'cpotp-cf7'],
        'subscribe_to_download_verify' => ['cpotp-subscribe-to-download', 'cpotp-subscribe-to-download'],
        'shortcode_login_verify' => ['cp-sc-fb-verify', 'cp-sc-fb-verify'],
    ];

    public function __construct()
    {
        foreach ($this->public as $callBack => $action) {
            add_action("wp_ajax_{$action[0]}", [$this, $callBack]);
            if (isset($action[1])) {
                add_action("wp_ajax_nopriv_{$action[1]}", [$this, $callBack]);
            }
        }
    }

    private function init_data()
    {
        $this->cp_otp_options = cp_otp_page_setting::get_options();

        if (empty($this->cp_otp_options['app_id'])) {
            $errors['app_id'] = __cp("Facebook app id is require");
        }
        if (empty($this->cp_otp_options['app_secret'])) {
            $errors['app_secret'] = __cp("Facebook app secret is require");
        }
        if (empty($this->cp_otp_options['app_version'])) {
            $errors['app_version'] = __cp("Facebook app version is require");
        }

        $errors = [];
        $code = isset($_GET['fb_code']) ? sanitize_text_field(trim($_GET['fb_code'])) : "";
        if (empty($code)) {
            $errors['fb_code'] = __cp("Facebook code login is require");
        }
        if (count($errors) > 0) {
            wp_send_json_error([
                'message' => __cp('Params request invalid'),
                'result' => $errors
            ]);
        }

        $resultExchange = $this->exchange_fb_code_login($code);

        if (!$resultExchange['success']) {
            wp_send_json_error([
                'message' => $resultExchange['message']
            ]);
        }

        $loginData = $this->get_login_data($resultExchange['result']['access_token']);
        if (!$loginData['success']) {
            wp_send_json_error([
                'message' => $loginData['message']
            ]);
        }

        $timeExpire = time() + 30 * DAY_IN_SECONDS;

        if (isset($loginData['result']['phone'])) {
            $phone = $loginData['result']['phone']['country_prefix'] . $loginData['result']['phone']['national_number'];
            $phone = "+" . $phone;

            setcookie('cpotp_phone', $phone, $timeExpire, COOKIEPATH, COOKIE_DOMAIN);
            setcookie('cpotp_phone_prefix', $loginData['result']['phone']['country_prefix'], $timeExpire, COOKIEPATH, COOKIE_DOMAIN);
        } elseif (isset($loginData['result']['email'])) {
            $email = $loginData['result']['email']['address'];
            setcookie('cpotp_email', $email, $timeExpire, COOKIEPATH, COOKIE_DOMAIN);
        }

        return $loginData;
    }


    public function woocommerce_download_verify()
    {
        global $wpdb;
        $object_type = 'woo_product';

        $product_id = \CastPlugin\CpCore::getRequest('product_id', ['int'], '');
        $product = wc_get_product($product_id);

        if (!$product) {
            wp_send_json_error([
                'message' => __cp("Product not found."),
                'result' => []
            ]);
        }

        $loginData = $this->init_data();

        $prefix_phone = $email = $phone = '';
        $user = false;

        if (isset($loginData['result']['phone'])) {
            $phone = $loginData['result']['phone']['country_prefix'] . $loginData['result']['phone']['national_number'];
            $prefix_phone = $loginData['result']['phone']['country_prefix'];
            $user = $this->get_user_by_phone($phone);
            $phone = "+" . $phone;

            if ($user['success'] == true) {
                $email = $user['result']->get('cpotp_email');
            }
        } elseif (isset($loginData['result']['email'])) {
            $email = $loginData['result']['email']['address'];
            $user = $this->get_user_by_email($email);
            if ($user) {
                $phone = $user['result']->get('cpotp_phone');
            }
        }

        $output = [];
        $output['html'] = cpsmart_otp_templates::get_table_download_woocommerce($product);

        wp_send_json_success(
            $output
        );
    }


    public function cf7_verify()
    {
        $loginData = $this->init_data();

        $cf7Data = isset($_GET['cf7_data']) ? $_GET['cf7_data'] : '';
        parse_str($cf7Data, $cf7Data);
        if (!isset($cf7Data['_wpcf7']) || empty($cf7Data['_wpcf7'])) {
            wp_send_json_error([
                'message' => __cp("Form ID not found.")
            ]);
        }

        $errors = [];

        $prefix_phone = $email = $phone = '';
        if (isset($loginData['result']['phone'])) {
            $phone = "+" . $loginData['result']['phone']['country_prefix'] . $loginData['result']['phone']['national_number'];
            $prefix_phone = $loginData['result']['phone']['country_prefix'];
        } elseif (isset($loginData['result']['email'])) {
            $email = $loginData['result']['email']['address'];
        } else {
            wp_send_json_error([
                'message' => __cp("Verification error! Please try again later."),
                'result' => []
            ]);
        }

        $userId = '';
        $user = wp_get_current_user();
        if (!is_wp_error($user) && isset($user->ID)) {
            $userId = $user->ID;
        }

        $idLog = cpsmart_otp_log::add_log_cf7([
            'phone' => $phone,
            'phone_prefix' => $prefix_phone,
            'form_id' => $cf7Data['_wpcf7'],
            'user_id' => $userId,
            'email' => $email,
            'referral' => isset($_GET['referral']) ? esc_url($_GET['referral']) : "",
        ]);

        wp_send_json_success([
            'message' => __cp("Verify success! You can submit form."),
            'result' => [
                'phone' => $phone,
                'email' => $email,
                'id' => $idLog
            ]
        ]);
    }

    public function woocommerce_verify()
    {
        $code = isset($_GET['fb_code']) ? sanitize_text_field(trim($_GET['fb_code'])) : "";
        $secret = isset($_GET['fb_secret']) ? sanitize_text_field(trim($_GET['fb_secret'])) : '';
        $order = isset($_GET['order']) ? $_GET['order'] : "";

        $loginData = $this->init_data();
        $errors = [];

        $prefix_phone = $email = $phone = '';

        $verifyStatus = false;

        if (isset($loginData['result']['phone'])) {
            $phone = "+" . $loginData['result']['phone']['country_prefix'] . $loginData['result']['phone']['national_number'];
            $prefix_phone = $loginData['result']['phone']['country_prefix'];

            cpsmart_otp_woo::update_order_by_phone($phone, $order['order_id']);
            $verifyStatus = true;
        } elseif (isset($loginData['result']['email'])) {
            $email = $loginData['result']['email']['address'];

            cpsmart_otp_woo::update_order_by_email($email, $order['order_id']);
            $verifyStatus = true;
        } else {
            wp_send_json_success([
                'message' => __cp("Verification error! Try again later."),
                'result' => []
            ]);
        }

        $orderObject = wc_get_order($order['order_id']);

        cpsmart_otp_log::add_log_woo([
            'phone' => $phone,
            'phone_prefix' => $prefix_phone,
            'order_id' => $orderObject->get_id(),
            'user_id' => $orderObject->get_user_id(),
            'email' => $email,
            'form_name' => isset($order['form_name']) ? sanitize_text_field(trim($order['form_name'])) : "",
            'referral' => isset($_GET['referral']) ? esc_url($_GET['referral']) : "",
        ]);

        wp_send_json_success();
    }


    private function exchange_fb_code_login($code)
    {
        if (!is_array($this->cp_otp_options)) {
            return false;
        }

        $app_id = $this->cp_otp_options['app_id'];
        $account_secret = $this->cp_otp_options['app_secret'];
        $account_kit_ver = $this->cp_otp_options['app_version'];

        $errors = [];
        //Exchange Code login
        $token_exchange_url = "https://graph.accountkit.com/{$account_kit_ver}/access_token?";
        $token_exchange_url .= http_build_query(array(
            "grant_type" => "authorization_code",
            "code" => $code,
            "access_token" => "AA|$app_id|$account_secret"
        ));

        $response = wp_remote_get($token_exchange_url);
        if (is_wp_error($response)) {
            return [
                "success" => false,
                'message' => $response->get_error_message()
            ];
        }

        $result = json_decode($response['body'], true);
        if (isset($result['error'])) {
            return [
                "success" => false,
                'message' => $result['error']['message'],
            ];
        }


        return [
            'success' => true,
            'result' => $result
        ];
    }

    private function get_login_data($accessToken)
    {
        $account_kit_ver = $this->cp_otp_options['app_version'];

        $me_endpoint_url = "https://graph.accountkit.com/{$account_kit_ver}/me?access_token={$accessToken}";
        $response = wp_remote_get($me_endpoint_url);
        if (is_wp_error($response)) {
            return [
                'success' => false,
                'message' => $response->get_error_message()
            ];
        }

        $result = json_decode($response['body'], true);
        if (isset($result['error'])) {
            return [
                'success' => false,
                'message' => $result['error']['message'],
            ];
        }

        return [
            'success' => true,
            'result' => $result
        ];
    }

    private function set_user_login($user)
    {
        if (is_user_logged_in()) {
            wp_clear_auth_cookie();
            clean_user_cache($user->ID);
        }

        wp_set_current_user($user->ID);
        wp_set_auth_cookie($user->ID, true);

        do_action('wp_login', $user->user_login, $user);

        update_user_caches($user);
    }

    private function get_user_by_phone($phone, $create = false)
    {
        if (!username_exists($phone)) {
            if (get_option('users_can_register') && $create) {
                $random_password = wp_generate_password($length = 12, $include_standard_special_chars = false);
                $user_id = wp_create_user($phone, $random_password);
                $user = get_user_by('ID', $user_id);
            } else {
                $user = false;

                return [
                    'success' => false,
                    'message' => __cp("Can't create new user. Please check your register setting on General setting page.")
                ];
            }
        } else {
            $user = get_user_by('login', $phone);
        }

        if ($user) {
            update_user_meta($user->ID, 'cpotp_phone', "+" . $phone);
        }

        return [
            'success' => true,
            'result' => $user
        ];
    }

    private function get_user_by_email($mail, $create = false)
    {
        if (!email_exists($mail)) {
            if (get_option('users_can_register') && $create) {
                $random_password = wp_generate_password($length = 12, $include_standard_special_chars = false);
                $user_id = wp_create_user($mail, $random_password, $mail);
                $user = get_user_by('ID', $user_id);
            } else {
                $user = false;
                return [
                    'success' => false,
                    'message' => __cp("Can't create new user. Please check your register setting on General setting page.")
                ];
            }
        } else {
            $user = get_user_by('email', $mail);
        }

        if ($user) {
            update_user_meta($user->ID, 'cpotp_email', $mail);
        }

        return [
            'success' => true,
            'result' => $user
        ];
    }

    public function facebook_verify()
    {
        $errors = [];

        $this->cp_otp_options = cp_otp_page_setting::get_options();
        if (empty($this->cp_otp_options['app_id'])) {
            $errors['app_id'] = __cp("Facebook app id is require");
        }
        if (empty($this->cp_otp_options['app_secret'])) {
            $errors['app_secret'] = __cp("Facebook app secret is require");
        }
        if (empty($this->cp_otp_options['app_version'])) {
            $errors['app_version'] = __cp("Facebook app version is require");
        }

        $code = isset($_GET['fb_code']) ? sanitize_text_field(trim($_GET['fb_code'])) : "";
        $secret = isset($_GET['fb_secret']) ? sanitize_text_field(trim($_GET['fb_secret'])) : '';

        if (empty($code)) {
            $errors['fb_code'] = __cp("Facebook code login is require");
        }

        if (count($errors) > 0) {
            wp_send_json_error([
                'message' => __cp('Params request invalid'),
                'result' => $errors
            ]);
        }

        $resultExchange = $this->exchange_fb_code_login($code);
        if (!$resultExchange['success']) {
            wp_send_json_error([
                'message' => $resultExchange['message']
            ]);
        }


        $loginData = $this->get_login_data($resultExchange['result']['access_token']);
        if (!$loginData['success']) {
            wp_send_json_error([
                'message' => $loginData['message']
            ]);
        }

        if (isset($loginData['result']['phone'])) {
            $phone = $loginData['result']['phone']['country_prefix'] . $loginData['result']['phone']['national_number'];


            $user = $this->get_user_by_phone($phone, true);
            if (!$user['success']) {
                wp_send_json_error([
                    'message' => $user['message']
                ]);
            } else {
                $this->set_user_login($user['result']);
            }

            cpsmart_otp_log::add_log_login([
                'phone' => "+" . $phone,
                'phone_prefix' => $loginData['result']['phone']['country_prefix'],
                'user_id' => $user['result']->ID,
                'email' => '',
                'form_name' => isset($_GET['form_name']) ? sanitize_text_field(trim($_GET['form_name'])) : "",
                'referral' => isset($_GET['referral']) ? esc_url($_GET['referral']) : "",
            ]);

            wp_send_json_success([
                'result' => $user['result']
            ]);
        } elseif (isset($loginData['result']['email'])) {
            $user = $this->get_user_by_email($loginData['result']['email']['address'], true);

            if (!$user['success']) {
                wp_send_json_error([
                    'message' => $user['message']
                ]);
            } else {
                $this->set_user_login($user['result']);
            }

            cpsmart_otp_log::add_log_login([
                'phone' => '',
                'email' => $loginData['result']['email']['address'],
                'form_name' => isset($_GET['form_name']) ? sanitize_text_field(trim($_GET['form_name'])) : "",
                'referral' => isset($_GET['referral']) ? esc_url($_GET['referral']) : "",
                'user_id' => $user['result']->ID,
            ]);

            wp_send_json_success([
                'result' => $user['result']
            ]);
        } else {
            wp_send_json_error([
                'message' => "Error code: 003XX. Exception error"
            ]);
        }
    }

    //v1.2
    public function subscribe_to_download_verify()
    {
        $loginData = $this->init_data();

        $object_id = isset($_GET['object_id']) ? $_GET['object_id'] : '';

        if(!$object_id){
            wp_send_json_error([
                'message' => __cp('Params request invalid')
            ]);
        }

        $prefix_phone = $email = $phone = '';
        if (isset($loginData['result']['phone'])) {
            $phone = "+" . $loginData['result']['phone']['country_prefix'] . $loginData['result']['phone']['national_number'];
            $prefix_phone = $loginData['result']['phone']['country_prefix'];
        } elseif (isset($loginData['result']['email'])) {
            $email = $loginData['result']['email']['address'];
        } else {
            wp_send_json_error([
                'message' => __cp("Verification error! Please try again later."),
                'result' => []
            ]);
        }

        $userId = '';
        $user = wp_get_current_user();
        if (!is_wp_error($user) && isset($user->ID)) {
            $userId = $user->ID;
        }

       cpsmart_otp_log::add_log_default([
            'form_name' => __cp('Subscribe to download'),
            'phone' => $phone,
            'phone_prefix' => $prefix_phone,
            'object_id' => $object_id,
            'user_id' => $userId,
            'email' => $email,
            'referral' => isset($_GET['referral']) ? esc_url($_GET['referral']) : "",
        ]);

       $output = [];
       $output['html'] = cpsmart_otp_templates::get_subscribe_to_download_files($object_id);

        wp_send_json_success(
            $output
        );
    }

    public function shortcode_login_verify()
    {
        $errors = [];

        $this->cp_otp_options = cp_otp_page_setting::get_options();
        if (empty($this->cp_otp_options['app_id'])) {
            $errors['app_id'] = __cp("Facebook app id is require");
        }
        if (empty($this->cp_otp_options['app_secret'])) {
            $errors['app_secret'] = __cp("Facebook app secret is require");
        }
        if (empty($this->cp_otp_options['app_version'])) {
            $errors['app_version'] = __cp("Facebook app version is require");
        }

        $code = isset($_GET['fb_code']) ? sanitize_text_field(trim($_GET['fb_code'])) : "";
        $secret = isset($_GET['fb_secret']) ? sanitize_text_field(trim($_GET['fb_secret'])) : '';
        $username = isset($_GET['username']) ? sanitize_text_field(trim($_GET['username'])) : '';
        $register = isset($_GET['register']) ? sanitize_text_field(trim($_GET['register'])) : '';

        if (empty($username)) {
            $errors['username'] = __cp("Username is require");
        }

        if(!validate_username($username)){
            $errors['username'] = __cp("Username is not valid");
        }

        if (empty($code)) {
            $errors['fb_code'] = __cp("Facebook code login is require");
        }

        if (count($errors) > 0) {
            wp_send_json_error([
                'message' => __cp('Params request invalid'),
                'result' => $errors
            ]);
        }

        $resultExchange = $this->exchange_fb_code_login($code);
        if (!$resultExchange['success']) {
            wp_send_json_error([
                'message' => $resultExchange['message']
            ]);
        }


        $loginData = $this->get_login_data($resultExchange['result']['access_token']);
        if (!$loginData['success']) {
            wp_send_json_error([
                'message' => $loginData['message']
            ]);
        }

        if (isset($loginData['result']['phone'])) {
            $phone = $loginData['result']['phone']['country_prefix'] . $loginData['result']['phone']['national_number'];
            $user = $this->get_user_shortcode_login($username, '', $phone, true);
            if (!$user['success']) {
                wp_send_json_error([
                    'message' => $user['message']
                ]);
            } else {
                $this->set_user_login($user['result']);
            }

            cpsmart_otp_log::add_log_login([
                'phone' => "+" . $phone,
                'phone_prefix' => $loginData['result']['phone']['country_prefix'],
                'user_id' => $user['result']->ID,
                'email' => '',
                'form_name' => isset($_GET['form_name']) ? sanitize_text_field(trim($_GET['form_name'])) : "",
                'referral' => isset($_GET['referral']) ? esc_url($_GET['referral']) : "",
            ]);

            wp_send_json_success([
                'result' => $user['result']
            ]);
        } elseif (isset($loginData['result']['email'])) {
            $user = $this->get_user_shortcode_login($username, $loginData['result']['email']['address'], '', $register);
            if (!$user['success']) {
                wp_send_json_error([
                    'message' => $user['message']
                ]);
            } else {
                $this->set_user_login($user['result']);
            }

            cpsmart_otp_log::add_log_login([
                'phone' => '',
                'email' => $loginData['result']['email']['address'],
                'form_name' => isset($_GET['form_name']) ? sanitize_text_field(trim($_GET['form_name'])) : "",
                'referral' => isset($_GET['referral']) ? esc_url($_GET['referral']) : "",
                'user_id' => $user['result']->ID,
            ]);

            wp_send_json_success([
                'result' => $user['result']
            ]);
        } else {
            wp_send_json_error([
                'message' => "Error code: 003XX. Exception error"
            ]);
        }
    }

    private function get_user_shortcode_login($username, $email = '', $phone = '', $create = false)
    {
        if (!username_exists($username)) {
            if (get_option('users_can_register') && $create) {
                $random_password = wp_generate_password($length = 12, $include_standard_special_chars = false);
                $user_id = wp_create_user($username, $random_password);
                $user = get_user_by('ID', $user_id);

                if($phone) {
                    update_user_meta($user->ID, 'cpotp_phone', "+" . $phone);
                }
                if($email) {
                    update_user_meta($user->ID, 'cpotp_email', $email);
                }

            } else {
                $user = false;

                return [
                    'success' => false,
                    'message' => __cp("Can't create new user. Please check your register setting on General setting page.")
                ];
            }
        } else {
            $user = get_user_by('login', $username);
            if ($email) {
                $cpotp_email = get_user_meta($user->ID, 'cpotp_email', 1);
                if($cpotp_email) {
                    if ($cpotp_email != $email) {
                        return [
                            'success' => false,
                            'message' => __cp("Email is not valid!!!")
                        ];
                    }
                }else{
                    return [
                        'success' => false,
                        'message' => __cp("That username is taken. Try another.")
                    ];
                }
            } else if ($phone) {
                $cpotp_phone = get_user_meta($user->ID, 'cpotp_phone', 1);
                if($cpotp_phone) {
                    if ($cpotp_phone != '+' . $phone) {
                        return [
                            'success' => false,
                            'message' => __cp("Phone is not valid!!!")
                        ];
                    }
                }else{
                    return [
                        'success' => false,
                        'message' => __cp("That username is taken. Try another.")
                    ];
                }
            }
        }

        return [
            'success' => true,
            'result' => $user
        ];
    }
}
