<?php

class cp_otp_gutenberg
{
    public function __construct(){

        add_action('enqueue_block_editor_assets', [$this, 'block_subscribe_to_download_admin_assets']);
        add_action('enqueue_block_assets', [$this, 'block_subscribe_to_download_assets']);
        add_filter( 'block_categories', [$this, 'add_custom_block_category'], 10, 2);

        add_action("wp_ajax_cpotp_subscribe_to_download", [$this, 'handle_download']);
        add_action("wp_ajax_nopriv_cpotp_subscribe_to_download", [$this, 'handle_download']);

        register_meta( 'post', 'cpOtp_subscribe_to_download', array(
            'show_in_rest' => true,
            'single' => true,
            'type' => 'string',
        ) );
    }

    public function block_subscribe_to_download_admin_assets(){

        wp_register_script(
            'cpotp-subscribe-to-download-js',
            plugins_url( 'admin/js/cp-otp-subscribe-to-download.js', dirname(__FILE__) ),
            array('wp-blocks', 'wp-i18n', 'wp-element', 'wp-components', 'wp-editor')
        );
        register_block_type( 'cp-smart-otp/gutenberg-subscribe-to-download', array('editor_script' => 'cpotp-subscribe-to-download-js') );

        global $post;

        wp_localize_script( 'cpotp-subscribe-to-download-js', 'CpOtp', array(
            'includes_url'  => includes_url(),
            'object_id' => $post->ID,
            'cp_name'   => 'CatsPlugins'
        ) );
    }

    public function block_subscribe_to_download_assets(){
        if(!is_admin()) {
            wp_enqueue_script('cpotp-process-subscribe-to-download');
            wp_localize_script( 'cpotp-process-subscribe-to-download', 'CpOtpGuten', ['countryCode' => cp_otp_helper::get_country_code()]);
        }
        wp_enqueue_style(
            'cpotp-subscribe-to-download-css',
            CPOTP_PUBLIC . '/css/cp-otp-subscribe-to-download.css',
            array('wp-edit-blocks')
        );
    }

    public function add_custom_block_category( $categories, $post ) {
        return array_merge(
            $categories,
            array(
                array(
                    'slug' => 'cp-smart-otp',
                    'title' => __cp( "Smart OTP"),
                ),
            )
        );
    }

    public function handle_download()
    {

        global $wpdb;
        $id = \CastPlugin\CpCore::getRequest('id', ['strip_tags', 'trim']);
        $object_id = \CastPlugin\CpCore::getRequest('object_id', ['strip_tags', 'trim']);

        $cpotp_phone = isset($_COOKIE['cpotp_phone']) ? $_COOKIE['cpotp_phone'] : '';
        $cpotp_email = isset($_COOKIE['cpotp_email']) ? $_COOKIE['cpotp_email'] : '';

        if (empty($cpotp_email) && empty($cpotp_phone)) {
            cpsmart_otp_templates::page_error(__cp("Access deny"));
        }

        if (empty($id)) {
            cpsmart_otp_templates::page_error(__cp("Result not found"));
        }

        $attach_ids = get_post_meta($object_id, 'cpOtp_subscribe_to_download', 1);

        if (!$attach_ids) {
            cpsmart_otp_templates::page_error(__cp("File not found"));
        }

        $attach_ids = json_decode($attach_ids);
        $file_path = false;
        foreach ($attach_ids as $item) {
            if ($id == $item->id) {
                $file_path = get_attached_file($item->value);
            }
        }

        if (!$file_path) {
            cpsmart_otp_templates::page_error(__cp("File not found"));
        }

        $user = false;
        if (username_exists($cpotp_phone)) {
            $user = get_user_by('login', $cpotp_phone);
        } elseif (email_exists($cpotp_email)) {
            $user = get_user_by('email', $cpotp_email);
        }

        if ($user) {
            $cpotp_email = get_user_meta($user->ID, 'cpotp_email', true);
            $cpotp_phone = get_user_meta($user->ID, 'cpotp_phone', true);
        }

        $record = cpsmart_otp_log::getDownLoadRow($object_id, $id, $cpotp_phone, $cpotp_email);

        if (!$record) {
            $datasave = [
                "token" => $id,
                "files" => $file_path,
                "object_id" => $object_id,
                "phone" => $cpotp_phone,
                "email" => $cpotp_email,
                "user_id" => ($user ? $user->ID : ''),
                "limit" => 0,
                "created_at" => time(),
                "updated_at" => time(),
                "count" => 1,
                "status" => cpsmart_otp_log::DOWNLOAD_ACTIVE,
            ];

            $wpdb->show_errors(true);
            $wpdb->insert(cpsmart_otp_log::get_download_table(), $datasave);
        } else {
            $wpdb->update(cpsmart_otp_log::get_download_table(), [
                'count' => $record->count + 1,
                'files' => $file_path,
                "phone" => $cpotp_phone,
                "email" => $cpotp_email,
                "user_id" => ($user ? $user->ID : ''),
            ], [
                'id' => $record->id
            ]);
        }

        $filename = basename($file_path);

        if (strstr($filename, '?')) {
            $filename = current(explode('?', $filename));
        }

        // Trigger download via one of the methods.
        CP_Download_Handler::download_file_force($file_path, $filename);
    }

}

new cp_otp_gutenberg();